/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/


#ifndef __H_FUNC__
#define __H_FUNC__

#include <stdbool.h>

/** @file
 * Generic function pointer definitions and data stuctures to do with
 * functions.
 * 
 * @todo create an object type that can hold functions and be able to run them.
 */


/*
 * This data structure holds a pointer to a function.
 *
 * A Func object must hold a pointer to a function and can hold a string,
 * representing its name.
 */


/*
 * Common types of function pointers:
 */


/**
 * Generic function.
 * 
 * @param 1 a void pointer to some data
 * @param 2...n a list of other parameters
 */
typedef void * (* SFuncPointer)(void *, ...);

typedef void (* VoidFunc)();

typedef void (* MethodFunc)(void *);

/**
 * A CompFunc represents a standard comparison function.
 * 
 * @param 1 object to compare
 * @param 2 object to compare
 * 
 * compare parameter 1 with parameter 2.
 */
typedef bool (* CompFunc)(void*, void*);

typedef char * (* CharFunc)();

#endif
