/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "Map.h"
#include <stdlib.h>


struct _SMapPrivate {
  size_t len; // Length
  SMapItem * items[]; // Use calloc to deal with memory.
};

void method_map_deinit (SMap * Self);

SMapItem * s_map_item_new (void * key, void * value) {
  SMapItem * self = malloc (sizeof (SMapItem));
  self->key = key;
  self->value = value;
  return self;
}

void s_map_item_free (SMapItem * self, MethodFunc free_key,
                                       MethodFunc free_value) {
  free_key (self->key);
  free_value (self->value);
  free (self);
}

SMap * s_map_new ( CompFunc comp_func, MethodFunc free_key,
                                       MethodFunc free_value) {
  SMap * self = malloc (sizeof (SMap));
  SMapClass * klass = malloc (sizeof (SMapClass));
  
  s_base_object_set_class ((SBaseObjectInstance *) self, klass);
  
  s_base_object_set_deinit_method ((SBaseObjectInstance *) self, method_map_deinit);
  
  self->priv = malloc (sizeof (SMapPrivate));
  
  self->priv->len = 0;
  
  klass->is_equal = comp_func;
  
  /* free_* functions need to be checked if they are null and set the pointer
   * to free or s_base_object_free ()... Have to decite which...
   */
  klass->free_key = free_key;
  klass->free_value = free_value;

  return self;
}

void s_map_free (SMap * self) {
  s_base_object_free (self);
}


void s_map_add (SMap * self ,void * key, void * value) {
  SMapItem ** items = self->priv->items;
  SMapItem * item = s_map_item_new (key, value);
  if (self->priv->len == 0) {
    items = realloc (items, sizeof(SMapItem));
    items[0] = item;
    self->priv->len++;
  } else {
    items = realloc (items, sizeof(SMapItem) * (self->priv->len + 1));
    items[self->priv->len-1] = item;
    self->priv->len++;
  }
}

void * s_map_get (SMap * self, void * key) {
  SMapItem ** items = self->priv->items;
  SMapClass * klass = (SMapClass *) s_base_object_get_class ((SBaseObjectInstance *) self);
  CompFunc comp_func = klass->is_equal;
  for (size_t i = 0; i < self->priv->len; i++) {
    bool is_item = comp_func (key, items[i]);
    if (is_item) {
      return items[i];
    }
  }
  return NULL;
}

void s_map_remove (SMap * self, void * key) {

}

void method_map_deinit (SMap * self) {
  
}
